@@ -13,12 +13,19 @@
 # limitations under the License.
 
 from substra.__version__ import __version__
-from substra.sdk import Client, exceptions, DEBUG_OWNER
-
+from substra.sdk import DEBUG_OWNER
+from substra.sdk import BackendType
+from substra.sdk import Client
+from substra.sdk import exceptions
+from substra.sdk import models
+from substra.sdk import schemas
 
 __all__ = [
-    '__version__',
-    'Client',
-    'exceptions',
-    'DEBUG_OWNER',
+    "__version__",
+    "Client",
+    "exceptions",
+    "DEBUG_OWNER",
+    "BackendType",
+    "schemas",
+    "models",
 ]
