@@ -12,12 +12,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from substra.sdk.client import Client
-from substra.sdk.utils import retry_on_exception
+from substra.sdk import models
+from substra.sdk import schemas
 from substra.sdk.backends.local.backend import DEBUG_OWNER
+from substra.sdk.client import Client
+from substra.sdk.config import BackendType
+from substra.sdk.utils import retry_on_exception
 
 __all__ = [
-    'Client',
-    'retry_on_exception',
-    'DEBUG_OWNER',
+    "Client",
+    "retry_on_exception",
+    "DEBUG_OWNER",
+    "BackendType",
+    "schemas",
+    "models",
 ]
