@@ -12,16 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-ALGO = 'algo'
-AGGREGATE_ALGO = 'aggregate_algo'
-COMPOSITE_ALGO = 'composite_algo'
-DATA_SAMPLE = 'data_sample'
-DATASET = 'dataset'
-MODEL = 'model'
-OBJECTIVE = 'objective'
-TESTTUPLE = 'testtuple'
-TRAINTUPLE = 'traintuple'
-AGGREGATETUPLE = 'aggregatetuple'
-COMPOSITE_TRAINTUPLE = 'composite_traintuple'
-COMPUTE_PLAN = 'compute_plan'
-NODE = 'node'
+ALGO = "algo"
+DATA_SAMPLE = "data_sample"
+DATASET = "dataset"
+MODEL = "model"
+METRIC = "metric"
+PREDICTTUPLE = "predicttuple"
+TESTTUPLE = "testtuple"
+TRAINTUPLE = "traintuple"
+AGGREGATETUPLE = "aggregatetuple"
+COMPOSITE_TRAINTUPLE = "composite_traintuple"
+COMPUTE_PLAN = "compute_plan"
+ORGANIZATION = "organization"
