@@ -15,7 +15,6 @@ import os
 
 from substra.sdk.hasher import Hasher
 
-
 _BLOCK_SIZE = 64 * 1024
 
 
@@ -23,7 +22,7 @@ def hash_file(path):
     """Hash a file."""
     hasher = Hasher()
 
-    with open(path, 'rb') as fp:
+    with open(path, "rb") as fp:
         while True:
             data = fp.read(_BLOCK_SIZE)
             if not data:
@@ -36,7 +35,7 @@ def hash_directory(path, followlinks=False):
     """Hash a directory."""
 
     if not os.path.isdir(path):
-        raise TypeError(f'{path} is not a directory.')
+        raise TypeError(f"{path} is not a directory.")
 
     hash_values = []
     for root, dirs, files in os.walk(path, topdown=True, followlinks=followlinks):
