@@ -18,8 +18,10 @@ import enum
 import json
 import pathlib
 import typing
-from typing import Optional, List, Dict
 import uuid
+from typing import Dict
+from typing import List
+from typing import Optional
 
 import pydantic
 
@@ -32,24 +34,68 @@ from substra.sdk import utils
 
 
 _SERVER_NAMES = {
-    'dataset': 'data_manager',
+    "dataset": "data_manager",
+}
+
+
+ALGO_INPUTS_PER_CATEGORY = {
+    "ALGO_SIMPLE": {
+        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "multiple": True, "optional": False},
+        "model": {"kind": "ASSET_MODEL", "multiple": False, "optional": True},
+        "opener": {"kind": "ASSET_DATA_MANAGER", "multiple": False, "optional": False},
+    },
+    "ALGO_AGGREGATE": {
+        "model": {"kind": "ASSET_MODEL", "multiple": True, "optional": False},
+    },
+    "ALGO_COMPOSITE": {
+        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "multiple": True, "optional": False},
+        "local": {"kind": "ASSET_MODEL", "multiple": False, "optional": True},
+        "opener": {"kind": "ASSET_DATA_MANAGER", "multiple": False, "optional": False},
+        "shared": {"kind": "ASSET_MODEL", "multiple": False, "optional": True},
+    },
+    "ALGO_METRIC": {
+        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "multiple": True, "optional": False},
+        "opener": {"kind": "ASSET_DATA_MANAGER", "multiple": False, "optional": False},
+        "predictions": {"kind": "ASSET_MODEL", "multiple": False, "optional": False},
+    },
+    "ALGO_PREDICT": {
+        "datasamples": {"kind": "ASSET_DATA_SAMPLE", "multiple": True, "optional": False},
+        "opener": {"kind": "ASSET_DATA_MANAGER", "multiple": False, "optional": False},
+        "model": {"kind": "ASSET_MODEL", "multiple": False, "optional": False},
+        "shared": {"kind": "ASSET_MODEL", "multiple": False, "optional": True},
+    },
+}
+
+ALGO_OUTPUTS_PER_CATEGORY = {
+    "ALGO_SIMPLE": {
+        "model": {"kind": "ASSET_MODEL", "multiple": False},
+    },
+    "ALGO_AGGREGATE": {
+        "model": {"kind": "ASSET_MODEL", "multiple": False},
+    },
+    "ALGO_COMPOSITE": {
+        "local": {"kind": "ASSET_MODEL", "multiple": False},
+        "shared": {"kind": "ASSET_MODEL", "multiple": False},
+    },
+    "ALGO_METRIC": {
+        "performance": {"kind": "ASSET_PERFORMANCE", "multiple": False},
+    },
+    "ALGO_PREDICT": {"predictions": {"kind": "ASSET_MODEL", "multiple": False}},
 }
 
 
 class Type(enum.Enum):
-    Algo = 'algo'
-    AggregateAlgo = 'aggregate_algo'
-    CompositeAlgo = 'composite_algo'
-    DataSample = 'data_sample'
-    Dataset = 'dataset'
-    Model = 'model'
-    Objective = 'objective'
-    Testtuple = 'testtuple'
-    Traintuple = 'traintuple'
-    Aggregatetuple = 'aggregatetuple'
-    CompositeTraintuple = 'composite_traintuple'
-    ComputePlan = 'compute_plan'
-    Node = 'node'
+    Algo = "algo"
+    DataSample = "data_sample"
+    Dataset = "dataset"
+    Model = "model"
+    Predicttuple = "predicttuple"
+    Testtuple = "testtuple"
+    Traintuple = "traintuple"
+    Aggregatetuple = "aggregatetuple"
+    CompositeTraintuple = "composite_traintuple"
+    ComputePlan = "compute_plan"
+    Organization = "organization"
 
     def to_server(self):
         """Returns the name used to identify the asset on the backend."""
@@ -60,16 +106,46 @@ class Type(enum.Enum):
         return self.name
 
 
+class AlgoCategory(str, enum.Enum):
+    """Algo category"""
+
+    unknown = "ALGO_UNKNOWN"
+    simple = "ALGO_SIMPLE"
+    composite = "ALGO_COMPOSITE"
+    aggregate = "ALGO_AGGREGATE"
+    metric = "ALGO_METRIC"
+    predict = "ALGO_PREDICT"
+
+
+class TaskCategory(str, enum.Enum):
+    """Task category"""
+
+    unknown = "TASK_UNKNOWN"
+    train = "TASK_TRAIN"
+    aggregate = "TASK_AGGREGATE"
+    composite = "TASK_COMPOSITE"
+    predict = "TASK_PREDICT"
+    test = "TASK_TEST"
+
+
 class _PydanticConfig(pydantic.BaseModel):
     """Shared configuration for all schemas here"""
+
     class Config:
-        # Raise an error for extra parameters at init
-        extra = 'forbid'
+        # Ignore extra fields, leave them unexposed
+        extra = "ignore"
 
 
 class _Spec(_PydanticConfig, abc.ABC):
     """Asset creation specification base class."""
 
+    # pretty print
+    def __str__(self):
+        return self.json(indent=4)
+
+    def __repr__(self):
+        return self.json(indent=4)
+
     class Meta:
         file_attributes = None
 
@@ -96,15 +172,17 @@ class Permissions(_PydanticConfig):
     """Specification for permissions. If public is False,
     give the list of authorized ids.
     """
+
     public: bool
-    authorized_ids: typing.List[str]  # List of authorized node ids if private
+    authorized_ids: typing.List[str]  # List of authorized organization ids if private
 
 
 class PrivatePermissions(_PydanticConfig):
-    """Specification for private permissions. Only the nodes whose
+    """Specification for private permissions. Only the organizations whose
     ids are in authorized_ids can access the asset.
     """
-    authorized_ids: typing.List[str]  # List of authorized node ids
+
+    authorized_ids: typing.List[str]  # List of authorized organization ids
 
 
 class DataSampleSpec(_Spec):
@@ -112,6 +190,7 @@ class DataSampleSpec(_Spec):
     To create one data sample, use the 'path' field, otherwise use
     the 'paths' field.
     """
+
     path: Optional[pathlib.Path]  # Path to the data sample if only one
     paths: Optional[List[pathlib.Path]]  # Path to the data samples if several
     test_only: bool  # If the data sample is for train or test
@@ -123,11 +202,11 @@ class DataSampleSpec(_Spec):
         return self.paths and len(self.paths) > 0
 
     @pydantic.root_validator(pre=True)
-    def exclusive_paths(cls, values):
+    def exclusive_paths(cls, values):  # noqa: N805
         """Check that one and only one path(s) field is defined."""
-        if 'paths' in values and 'path' in values:
+        if "paths" in values and "path" in values:
             raise ValueError("'path' and 'paths' fields are exclusive.")
-        if 'paths' not in values and 'path' not in values:
+        if "paths" not in values and "path" not in values:
             raise ValueError("'path' or 'paths' field must be set.")
         return values
 
@@ -143,271 +222,307 @@ class DataSampleSpec(_Spec):
             yield data, None
 
 
-class ComputePlanTraintupleSpec(_Spec):
+class _ComputePlanComputeTaskSpec(_Spec):
+    """Specification of a compute task inside a compute plan specification"""
+
+    algo_key: str
+    tag: Optional[str]
+    metadata: Optional[Dict[str, str]]
+
+
+class ComputePlanTraintupleSpec(_ComputePlanComputeTaskSpec):
     """Specification of a traintuple inside a compute
     plan specification"""
-    algo_key: str
+
     data_manager_key: str
     train_data_sample_keys: List[str]
     traintuple_id: str
     in_models_ids: Optional[List[str]]
-    tag: Optional[str]
-    metadata: Optional[Dict[str, str]]
 
 
-class ComputePlanAggregatetupleSpec(_Spec):
+class ComputePlanAggregatetupleSpec(_ComputePlanComputeTaskSpec):
     """Specification of an aggregate tuple inside a compute
     plan specification"""
+
     aggregatetuple_id: str
-    algo_key: str
     worker: str
     in_models_ids: Optional[List[str]]
-    tag: Optional[str]
-    metadata: Optional[Dict[str, str]]
 
 
-class ComputePlanCompositeTraintupleSpec(_Spec):
+class ComputePlanCompositeTraintupleSpec(_ComputePlanComputeTaskSpec):
     """Specification of a composite traintuple inside a compute
     plan specification"""
+
     composite_traintuple_id: str
-    algo_key: str
     data_manager_key: str
     train_data_sample_keys: List[str]
     in_head_model_id: Optional[str]
     in_trunk_model_id: Optional[str]
-    tag: Optional[str]
     out_trunk_model_permissions: Permissions
-    metadata: Optional[Dict[str, str]]
 
 
-class ComputePlanTesttupleSpec(_Spec):
+class ComputePlanPredicttupleSpec(_ComputePlanComputeTaskSpec):
+    """Specification of a predict tuple inside a compute
+    plan specification"""
+
+    predicttuple_id: str
+    traintuple_id: str
+    data_manager_key: str
+    test_data_sample_keys: List[str]
+
+
+class ComputePlanTesttupleSpec(_ComputePlanComputeTaskSpec):
     """Specification of a testtuple inside a compute
     plan specification"""
-    objective_key: str
-    traintuple_id: str
-    tag: Optional[str]
-    data_manager_key: Optional[str]
-    test_data_sample_keys: Optional[List[str]]
-    metadata: Optional[Dict[str, str]]
+
+    predicttuple_id: str
+    data_manager_key: str
+    test_data_sample_keys: List[str]
 
 
 class _BaseComputePlanSpec(_Spec, abc.ABC):
+    key: str
     traintuples: Optional[List[ComputePlanTraintupleSpec]]
     composite_traintuples: Optional[List[ComputePlanCompositeTraintupleSpec]]
     aggregatetuples: Optional[List[ComputePlanAggregatetupleSpec]]
+    predicttuples: Optional[List[ComputePlanPredicttupleSpec]]
     testtuples: Optional[List[ComputePlanTesttupleSpec]]
 
 
 class ComputePlanSpec(_BaseComputePlanSpec):
     """Specification for creating a compute plan"""
+
+    key: str = pydantic.Field(default_factory=lambda: str(uuid.uuid4()))
     tag: Optional[str]
+    name: str
     clean_models: Optional[bool]
     metadata: Optional[Dict[str, str]]
 
     type_: typing.ClassVar[Type] = Type.ComputePlan
 
+    @contextlib.contextmanager
+    def build_request_kwargs(self):
+        # default values are not dumped when `exclude_unset` flag is enabled,
+        # this is why we need to reimplement this custom function.
+        data = json.loads(self.json(exclude_unset=True))
+        data["key"] = self.key
+        yield data, None
+
 
 class UpdateComputePlanSpec(_BaseComputePlanSpec):
     """Specification for updating a compute plan"""
+
     pass
 
 
 class DatasetSpec(_Spec):
     """Specification for creating a dataset"""
+
     name: str
     data_opener: pathlib.Path  # Path to the data opener
     type: str
     description: pathlib.Path  # Path to the description file
     permissions: Permissions
-    objective_key: Optional[str]
     metadata: Optional[Dict[str, str]]
+    logs_permission: Permissions
 
     type_: typing.ClassVar[Type] = Type.Dataset
 
     class Meta:
-        file_attributes = ('data_opener', 'description', )
+        file_attributes = (
+            "data_opener",
+            "description",
+        )
 
 
-class ObjectiveSpec(_Spec):
-    """Specification for creating an objective"""
-    name: str
-    description: pathlib.Path  # Path to the description file
-    metrics_name: str
-    metrics: pathlib.Path  # Path to the metrics file
-    test_data_sample_keys: Optional[List[str]]
-    test_data_manager_key: Optional[str]
-    permissions: Permissions
-    metadata: Optional[Dict[str, str]]
+class AlgoSpec(_Spec):
+    """Specification for creating an algo"""
 
-    type_: typing.ClassVar[Type] = Type.Objective
-
-    class Meta:
-        file_attributes = ('metrics', 'description', )
-
-
-class _AlgoSpec(_Spec):
     name: str
     description: pathlib.Path
     file: pathlib.Path
     permissions: Permissions
     metadata: Optional[Dict[str, str]]
+    category: AlgoCategory
 
-    class Meta:
-        file_attributes = ('file', 'description', )
-
-
-class AlgoSpec(_AlgoSpec):
-    """Specification for creating an algo"""
     type_: typing.ClassVar[Type] = Type.Algo
 
+    @contextlib.contextmanager
+    def build_request_kwargs(self):
+        # TODO should be located in the backends/remote module
+        # Serialize and deserialize to prevent errors eg with pathlib.Path
+        data = json.loads(self.json(exclude_unset=True))
 
-class AggregateAlgoSpec(_AlgoSpec):
-    """Specification for creating an aggregate algo"""
-    type_: typing.ClassVar[Type] = Type.AggregateAlgo
+        # Waiting for algo inputs/outputs to be added by the public API,
+        # these data are added on the fly to the request, without being exposed to the user.
+        # Computed fields using `@property` are not dumped when `exclude_unset` flag is enabled,
+        # this is why we need to reimplement this custom function.
+        data["inputs"] = ALGO_INPUTS_PER_CATEGORY[self.category]
+        data["outputs"] = ALGO_OUTPUTS_PER_CATEGORY[self.category]
 
+        if self.Meta.file_attributes:
+            with utils.extract_files(data, self.Meta.file_attributes) as (data, files):
+                yield (data, files)
+        else:
+            yield data, None
 
-class CompositeAlgoSpec(_AlgoSpec):
-    """Specification for creating a composite algo"""
-    type_: typing.ClassVar[Type] = Type.CompositeAlgo
-
-
-class TraintupleSpec(_Spec):
-    """Specification for creating a traintuple"""
-    algo_key: str
-    data_manager_key: str
-    train_data_sample_keys: List[str]
-    in_models_keys: Optional[List[str]]
-    tag: Optional[str]
-    compute_plan_key: Optional[str]
-    rank: Optional[int]  # Rank of the traintuple in the compute plan
-    metadata: Optional[Dict[str, str]]
-
-    compute_plan_attr_name: typing.ClassVar[str] = "traintuple_keys"
-    algo_type: typing.ClassVar[Type] = Type.Algo
-    type_: typing.ClassVar[Type] = Type.Traintuple
-
-    @classmethod
-    def from_compute_plan(
-        cls,
-        compute_plan_key: str,
-        id_to_key: typing.Dict[str, str],
-        rank: int,
-        spec: ComputePlanTraintupleSpec
-    ) -> "TraintupleSpec":
-        return TraintupleSpec(
-            algo_key=spec.algo_key,
-            data_manager_key=spec.data_manager_key,
-            train_data_sample_keys=spec.train_data_sample_keys,
-            in_models_keys=[
-                id_to_key[parent_id] for parent_id in spec.in_models_ids
-            ] if spec.in_models_ids is not None else list(),
-            tag=spec.tag,
-            compute_plan_key=compute_plan_key,
-            rank=rank,
-            metadata=spec.metadata
+    class Meta:
+        file_attributes = (
+            "file",
+            "description",
         )
 
 
-class AggregatetupleSpec(_Spec):
-    """Specification for creating an aggregate tuple"""
-    algo_key: str
-    worker: str
-    in_models_keys: List[str]
+class _TupleSpec(_Spec):
+    key: str = pydantic.Field(default_factory=lambda: str(uuid.uuid4()))
     tag: Optional[str]
     compute_plan_key: Optional[str]
-    rank: Optional[int]
     metadata: Optional[Dict[str, str]]
+    algo_key: str
+
+    @contextlib.contextmanager
+    def build_request_kwargs(self):
+        # default values are not dumped when `exclude_unset` flag is enabled,
+        # this is why we need to reimplement this custom function.
+        data = json.loads(self.json(exclude_unset=True))
+        data["key"] = self.key
+        data["category"] = self.category
+        yield data, None
+
+
+class TraintupleSpec(_TupleSpec):
+    """Specification for creating a traintuple"""
+
+    data_manager_key: str
+    train_data_sample_keys: List[str]
+    in_models_keys: Optional[List[str]]
+    rank: Optional[int]  # Rank of the traintuple in the compute plan
+    category: TaskCategory = pydantic.Field(TaskCategory.train, const=True)
+
+    compute_plan_attr_name: typing.ClassVar[str] = "traintuple_keys"
+    type_: typing.ClassVar[Type] = Type.Traintuple
+
+    @classmethod
+    def from_compute_plan(cls, compute_plan_key: str, rank: int, spec: ComputePlanTraintupleSpec) -> "TraintupleSpec":
+        return TraintupleSpec(
+            key=spec.traintuple_id,
+            algo_key=spec.algo_key,
+            data_manager_key=spec.data_manager_key,
+            train_data_sample_keys=spec.train_data_sample_keys,
+            in_models_keys=spec.in_models_ids or list(),
+            tag=spec.tag,
+            compute_plan_key=compute_plan_key,
+            rank=rank,
+            metadata=spec.metadata,
+        )
+
+
+class AggregatetupleSpec(_TupleSpec):
+    """Specification for creating an aggregate tuple"""
+
+    worker: str
+    in_models_keys: List[str]
+    rank: Optional[int]
+    category: TaskCategory = pydantic.Field(TaskCategory.aggregate, const=True)
 
     compute_plan_attr_name: typing.ClassVar[str] = "aggregatetuple_keys"
-    algo_type: typing.ClassVar[Type] = Type.AggregateAlgo
     type_: typing.ClassVar[Type] = Type.Aggregatetuple
 
     @classmethod
     def from_compute_plan(
-        cls,
-        compute_plan_key: str,
-        id_to_key: typing.Dict[str, str],
-        rank: int,
-        spec: ComputePlanAggregatetupleSpec
+        cls, compute_plan_key: str, rank: int, spec: ComputePlanAggregatetupleSpec
     ) -> "AggregatetupleSpec":
         return AggregatetupleSpec(
+            key=spec.aggregatetuple_id,
             algo_key=spec.algo_key,
             worker=spec.worker,
-            in_models_keys=[
-                id_to_key[parent_id]
-                for parent_id in spec.in_models_ids
-            ],
+            in_models_keys=spec.in_models_ids or list(),
             tag=spec.tag,
             compute_plan_key=compute_plan_key,
             rank=rank,
-            metadata=spec.metadata
+            metadata=spec.metadata,
         )
 
 
-class CompositeTraintupleSpec(_Spec):
+class CompositeTraintupleSpec(_TupleSpec):
     """Specification for creating a composite traintuple"""
-    algo_key: str
+
     data_manager_key: str
     train_data_sample_keys: List[str]
     in_head_model_key: Optional[str]
     in_trunk_model_key: Optional[str]
-    tag: Optional[str]
-    compute_plan_key: Optional[str]
-    out_trunk_model_permissions: PrivatePermissions
+    out_trunk_model_permissions: Permissions
     rank: Optional[int]
-    metadata: Optional[Dict[str, str]]
+    category: TaskCategory = pydantic.Field(TaskCategory.composite, const=True)
 
     compute_plan_attr_name: typing.ClassVar[str] = "composite_traintuple_keys"
     type_: typing.ClassVar[Type] = Type.CompositeTraintuple
 
     @classmethod
     def from_compute_plan(
-        cls,
-        compute_plan_key: str,
-        id_to_key: typing.Dict[str, str],
-        rank: int,
-        spec: ComputePlanCompositeTraintupleSpec
+        cls, compute_plan_key: str, rank: int, spec: ComputePlanCompositeTraintupleSpec
     ) -> "CompositeTraintupleSpec":
         return CompositeTraintupleSpec(
+            key=spec.composite_traintuple_id,
             algo_key=spec.algo_key,
             data_manager_key=spec.data_manager_key,
             train_data_sample_keys=spec.train_data_sample_keys,
-            in_head_model_key=(id_to_key[spec.in_head_model_id]
-                               if spec.in_head_model_id else None),
-            in_trunk_model_key=(id_to_key[spec.in_trunk_model_id]
-                                if spec.in_trunk_model_id else None),
+            in_head_model_key=spec.in_head_model_id,
+            in_trunk_model_key=spec.in_trunk_model_id,
             out_trunk_model_permissions={
-                "authorized_ids": spec.out_trunk_model_permissions.authorized_ids
+                "public": spec.out_trunk_model_permissions.public,
+                "authorized_ids": spec.out_trunk_model_permissions.authorized_ids,
             },
             tag=spec.tag,
             compute_plan_key=compute_plan_key,
             rank=rank,
-            metadata=spec.metadata
+            metadata=spec.metadata,
         )
 
 
-class TesttupleSpec(_Spec):
-    """Specification for creating a testtuple"""
-    objective_key: str
+class PredicttupleSpec(_TupleSpec):
+    """Specification for creating a predict tuple"""
+
     traintuple_key: str
-    tag: Optional[str]
-    data_manager_key: Optional[str]
-    test_data_sample_keys: Optional[List[str]]
-    metadata: Optional[Dict[str, str]]
+    data_manager_key: str
+    test_data_sample_keys: List[str]
+    category: TaskCategory = pydantic.Field(TaskCategory.predict, const=True)
+
+    compute_plan_attr_name: typing.ClassVar[str] = "predicttuple_keys"
+    type_: typing.ClassVar[Type] = Type.Predicttuple
+
+    @classmethod
+    def from_compute_plan(cls, compute_plan_key: str, spec: ComputePlanPredicttupleSpec) -> "PredicttupleSpec":
+        return PredicttupleSpec(
+            key=spec.predicttuple_id,
+            algo_key=spec.algo_key,
+            traintuple_key=spec.traintuple_id,
+            tag=spec.tag,
+            data_manager_key=spec.data_manager_key,
+            test_data_sample_keys=spec.test_data_sample_keys,
+            compute_plan_key=compute_plan_key,
+            metadata=spec.metadata,
+        )
+
+
+class TesttupleSpec(_TupleSpec):
+    """Specification for creating a testtuple"""
+
+    predicttuple_key: str
+    data_manager_key: str
+    test_data_sample_keys: List[str]
+    category: TaskCategory = pydantic.Field(TaskCategory.test, const=True)
 
     type_: typing.ClassVar[Type] = Type.Testtuple
 
     @classmethod
-    def from_compute_plan(
-        cls,
-        id_to_key: typing.Dict[str, str],
-        spec: ComputePlanTesttupleSpec
-    ) -> "TesttupleSpec":
+    def from_compute_plan(cls, compute_plan_key: str, spec: ComputePlanTesttupleSpec) -> "TesttupleSpec":
         return TesttupleSpec(
-            objective_key=spec.objective_key,
-            traintuple_key=id_to_key[spec.traintuple_id],
+            algo_key=spec.algo_key,
+            predicttuple_key=spec.predicttuple_id,
             tag=spec.tag,
             data_manager_key=spec.data_manager_key,
             test_data_sample_keys=spec.test_data_sample_keys,
+            compute_plan_key=compute_plan_key,
             metadata=spec.metadata,
         )
