@@ -12,42 +12,31 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from substra.sdk import models, schemas
+from substra.sdk import models
+from substra.sdk import schemas
 
 from .. import datastore
-from .utils import mock_requests
+from ..utils import mock_requests
 
 
-def test_update_dataset(client, mocker):
-    m = mock_requests(mocker, "post", response={"key": "dataset_key"})
-
-    response = client.link_dataset_with_objective(
-        'a key',
-        'an another key',
-    )
-
-    assert response == "dataset_key"
-    m.assert_called()
-
-
-def test_update_compute_plan(client, mocker):
+def test_add_compute_plan_tuples(client, mocker):
     item = datastore.COMPUTE_PLAN
     m = mock_requests(mocker, "post", response=item)
     m_get = mock_requests(mocker, "get", response=datastore.COMPUTE_PLAN)
 
-    response = client.update_compute_plan('foo', {})
+    response = client.add_compute_plan_tuples("foo", {})
 
     assert response == models.ComputePlan(**item)
     m.assert_called
     m_get.assert_called
 
 
-def test_update_compute_plan_with_schema(client, mocker):
+def test_add_compute_plan_tuples_with_schema(client, mocker):
     item = datastore.COMPUTE_PLAN
     m = mock_requests(mocker, "post", response=item)
     m_get = mock_requests(mocker, "get", response=datastore.COMPUTE_PLAN)
 
-    response = client.update_compute_plan('foo', schemas.UpdateComputePlanSpec())
+    response = client.add_compute_plan_tuples("foo", schemas.UpdateComputePlanSpec(key="foo"))
 
     assert response == models.ComputePlan(**item)
     m.assert_called
